// Copyright 2009 - Paul S. Heidmann
//
// Unless changed, in writing, by Paul S. Heidmann, this code is
// governed by the GPL, version 3. The terms of this copyright
// are found in the file throughputTest.cpp, included in this
// distribution.
//

#include "contentionPoint.hpp"

#include "timeProfileConsts.hpp"

contentionPoint::contentionPoint( const gaussianRandomVariable& produceWorkVar,
				  const gaussianRandomVariable& consumeWorkVar,
				  const unsigned long capacity,
				  const unsigned testTimeInSeconds ) :
  testTime( testTimeInSeconds ),
  produceWork( produceWorkVar ),
  consumeWork( consumeWorkVar ),
  testDone( false ),
  testStarted( false ),
  numConsumptions( 0 ),
  numItemsAvailable( 0 ),
  size( capacity )
{
}

contentionPoint::~contentionPoint()
{
}

void contentionPoint::startTest( void )
{
  this->startTime = boost::posix_time::second_clock::local_time();
  this->testStarted = true;
}

const bool contentionPoint::produce( timeProfile& p )
{
  p.insertProfilePoint( CONTENTION_POINT_PRODUCE | 1 );

  boost::mutex::scoped_lock localLock( this->lock );
  if( !(this->testStarted) )
    return( true );
  if( this->checkTestDone() )
    return( false );
  if( this->numItemsAvailable > this->size )
    return( true );

  p.insertProfilePoint( CONTENTION_POINT_PRODUCE | 2 );

  this->produceWork.doWork( p );
  ++(this->numItemsAvailable);

  p.insertProfilePoint( CONTENTION_POINT_PRODUCE | 3 );

  return( true );
}

const bool contentionPoint::consume( timeProfile& p )
{
  p.insertProfilePoint( CONTENTION_POINT_CONSUME | 1 );

  boost::mutex::scoped_lock localLock( this->lock );
  if( !(this->testStarted) )
    return( true );
  if( this->checkTestDone() )
    return( false );
  if( this->numItemsAvailable == 0 )
    return( true );

  p.insertProfilePoint( CONTENTION_POINT_CONSUME | 2 );

  this->consumeWork.doWork( p );
  ++(this->numConsumptions);
  --(this->numItemsAvailable);

  p.insertProfilePoint( CONTENTION_POINT_CONSUME | 3 );

  return( true );
}

const unsigned long contentionPoint::getNumConsumptions( void ) const
{
  return( this->numConsumptions );
}

const bool contentionPoint::checkTestDone( void )
{
  if( this->testDone )
    return( true );
  const boost::posix_time::ptime
    curTime( boost::posix_time::second_clock::local_time() );
  const boost::posix_time::time_duration
    diffTime( curTime - this->startTime );
  if( diffTime.total_seconds() > this->testTime )
  {
    this->testDone = true;
    return( true );
  }
  return( false );
}
